<?php

namespace Service\Integration;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Teknisa\ACL\Util\InstanceProvider;
use Util\Messages;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class PMIntegration {

    const CLIENT_ID = "CustomerPortal";
    const CLIENT_SECRET = "8c673f38ade42e5e3318c830d9522bd2";
    const ROUTE_LINK_BUDGET = '/linkDownloadBudgetSm';

    protected $entityManager;
    private $token;
    protected $PM_URL;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Task  */
    protected $taskRepository ;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Project  */
    protected $projectRepository ;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Attachment  */
    protected $attachmentRepository ;
    public $fileServerFilesUrl;
    /** @var \Util\Utilities */
    private $environment;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Comments */
    protected $commentsRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource */
    protected $resourceRepository;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, $PM_URL, $fileServerFilesUrl){
        $this->entityManager = $entityManager;
        $this->PM_URL = $PM_URL;
        $this->taskRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Task');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Project');
        $this->attachmentRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Attachment');
        $this->commentsRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Comments');
        $this->resourceRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Resource');
        $this->fileServerFilesUrl = $fileServerFilesUrl;
        $this->environment = $environment;
    }

    public function requestPMRoute($uri, $data = array()){
        if(empty($this->token)){
            $this->token = $this->requestToken();
        }
        $data = json_encode($data);
        $url = $this->PM_URL.$uri;
        $header = array();
        $header[] = 'OAuth-Token: '.$this->token;
        $header[] = 'OAuth-SecretId: '.self::CLIENT_SECRET;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Token-login: '.$this->environment->getToken();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: '.strlen($data);
        $header[] = 'Expect:';
        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);
        curl_close($curl_session);

        $result = json_decode($result, true);
        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        }  else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }

        return $result;
    }

    private function requestToken(){
        $data = array(
            "CLIENT_ID" => self::CLIENT_ID,
            "CLIENT_SECRET" => self::CLIENT_SECRET
        );

        $data = json_encode($data);
        $url = $this->PM_URL.'/requestToken';
        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Length: '.strlen($data);

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);

        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);

        curl_close($curl_session);
        $result = json_decode($result, true);
        $token = $result["dataset"]["token"][0];

        return $token;
    }

    private function getDataRequestFilter($filter = array(), $itemsPerPage = 500, $page = 1){
        $dataFilter = array(
            "requestType" => Request::TYPE_FILTER,
            "filter" => $filter,
            "itemsPerPage" => $itemsPerPage,
            "page" => $page
        );

        return $dataFilter;
    }

    private function getDataRequestRow($row) {
        $dataRow = array(
            "requestType" => Request::TYPE_ROW,
            "row" => $row
        );

        return $dataRow;
    }

    private function getDataRequestDataSet($dataSet) {
        $dataSet = array(
            "requestType" => Request::TYPE_DATA_SET,
            "dataset" => $dataSet
        );

        return $dataSet;
    }

     public function findIssueById($issueId){
        return $this->taskRepository->findOneBy(array("ID" => $issueId));
    }

    public function findProductCallClassification($params = array(), $itemsPerPage = 500, $page = 1){
        $filterString = DevOpsUtilities::createQueryFiltered($params);
        $classificationFilter = explode(',', \Util\Utilities::getClassificationFilter());
        $nrOrg          = \Util\Utilities::getEnvironment()->getCurrentOrganizationId();
        return $this->taskRepository->findIssueClassificationCP($nrOrg, $filterString, $classificationFilter);
    }

    public function findProductCallProducts($filter = array(), $itemsPerPage = 500, $page = 1)
    {
        $filterString = DevOpsUtilities::createQueryFiltered($filter, 'P');
        $organization = $this->environment->getPmCurrentOrganizationId();
        if(!$this->environment->currentUserIsTeknisa()){
            $filterUserAcess = "AND PRD.PRODUCT_TYPE = 'C'";
        }
        else{
            $filterUserAcess = "";
        }
        $products = $this->projectRepository->findProducts($organization, 0, false, false, false, $filterString,$filterUserAcess);
        $notModelProducts = DevOpsUtilities::removeModels($products, $organization);
        return $notModelProducts;
    }

    public function getLinkDownloadBudgetSm($issueId)
    {
        try{
            $dataRow = $this->getDataRequestRow(array('ISSUE_ID' => $issueId));
            $linkDownloadBudgetSm = $this->requestPMRoute(self::ROUTE_LINK_BUDGET, $dataRow);
            if(isset($linkDownloadBudgetSm['linkDownloadBudgetSm']) ){
                $linkDownloadBudgetSm = $linkDownloadBudgetSm['linkDownloadBudgetSm'][0];
            }
        } catch (\Exception $e) {
            throw new CustomException('ERR_BUDGET_INTEGRATION_PM', 500, array( 'error' => $e->getMessage()));
        }
        return $linkDownloadBudgetSm;
    }

    public function sendCommentsApplyDeploy($server, $version, $issues){
        $data = array(
            "SERVER"  => $server,
            "VERSION" => $version,
            "ISSUES"  => $issues,
            "USER_ID" => $this->environment->getPmCurrentUserId()
        );
        $errorLog = $this->requestPMRoute('/api/sendCommentsApplyDeploy', $this->getDataRequestDataSet([$data]));

        return $errorLog;
    }

    public function sendProductApplyDeploy($server, $product){
        $data = array(
            "SERVER"  => $server,
            "PRODUCT" => $product,
            "USER_ID" => $this->environment->getPmCurrentUserId()
        );
        $errorLog = $this->requestPMRoute('/api/sendProductApplyDeploy', $this->getDataRequestDataSet([$data]));

        return $errorLog;
    }

    public function getCurrentResource()
    {
        $cdoperador = $this->environment->getCurrentUser();
        $resource = $this->resourceRepository->findOneBy(array('externalCode' => $cdoperador));
        return $resource->getId();
    }

    public function createIssueByProductCall($reference, $product, $classification, $description, $board, $reporterName,
                                             $reporterFrom,$sponsorId,$reportedBy) {
        $description = $reporterName . " - " . $reporterFrom . PHP_EOL . PHP_EOL . PHP_EOL. $description;

        $now = new \DateTime();
        $now = $now->format('d/m/Y');

        $row = array(
            'REFERENCE' => $reference,
            'PRODUCT' => $product,
            'CLASSIFICATION' => $classification,
            'DESCRIPTION' => $description,
            'DUE_DATE' => $now,
            'SPONSOR' => $sponsorId,
            'BOARD' => $board ? $board : $product,
            'REPORTER_CDOPERADOR' => $reportedBy
        );

        $dataRow = $this->getDataRequestRow($row);
        $issue = $this->requestPMRoute('/createIssueByProductCall', $dataRow);

        if(empty($issue['issue'][0])){
            return array();
        } else {
            return $issue['issue'][0];
        }
    }

    public function getIssuesLastEvolutionDates($issuesIds) {

        $lastEvolutionDates = $this->lastEvolutionDates($issuesIds);

        $dates = array();

        if(!empty($lastEvolutionDates)){
            foreach($lastEvolutionDates as $lastEvolutionDate) {
                $dates[$lastEvolutionDate['ID']] = $lastEvolutionDate['LASTDATE'];
            }
        }

        return $dates;
    }

    public function updateIssueByProductCall($issueId, $reference, $product, $classification, $description, $board,
                                             $reporterName, $reporterFrom) {

        $description = $reporterName . " - " . $reporterFrom . PHP_EOL . PHP_EOL . PHP_EOL. $description;

        $now = new \DateTime();
        $now = $now->format('d/m/Y');

        $row = array(
            'ID' => $issueId,
            'REFERENCE' => $reference,
            'PRODUCT' => $product,
            'CLASSIFICATION' => $classification,
            'DESCRIPTION' => $description,
            'DUE_DATE' => $now,
            'BOARD' => $board
        );
        $dataRow = $this->getDataRequestRow($row);
        $issue = $this->requestPMRoute('/updateIssueByProductCall', $dataRow);

        if(empty($issue['issue'][0])){
            return array();
        } else {
            return $issue['issue'][0];
        }
    }

    public function removeIssueById($issueId) {
        $filter = array(
            array(
                "name" => "ISSUE_ID",
                "value" => $issueId,
                "operator" => "="
            )
        );

        $dataFilter = $this->getDataRequestFilter($filter);
        $result = $this->requestPMRoute('/removeIssueById', $dataFilter);
        return $result['removeIssueById'][0];
    }

    public function saveCommentOnIssueByProductCallComment($issueId, $comment, $createdBy, $cdCliente)
    {
        $row = array(
            "ISSUE_ID" => $issueId,
            "COMMENT" => $comment,
            "CDOPERADOR" => $createdBy,
            "CDCLIENTE" => $cdCliente,
        );

        return $this->requestPMRoute('/saveCommentOnIssueByProductCallComment', $this->getDataRequestRow($row));
    }

    public function saveReopenIssueByProductCall($issueId, $comment, $createdBy)
    {
        $row = array(
            "ISSUE_ID" => $issueId,
            "COMMENT" => $comment,
            "CDOPERADOR" => $createdBy,
        );

        return $this->requestPMRoute('/saveReopenIssueByProductCall', $this->getDataRequestRow($row));
    }

    public function saveRefuseIssueByProductCall($issueId, $comment, $createdBy)
    {
        $row = array(
            "ISSUE_ID" => $issueId,
            "COMMENT" => $comment,
            "CDOPERADOR" => $createdBy,
        );
        return $this->requestPMRoute('/saveRefuseIssueByProductCall', $this->getDataRequestRow($row));
    }

    public function saveApproveIssueByProductCall($issueId, $comment, $createdBy,$document = null)
    {
        $row = array(
            "ISSUE_ID" => $issueId,
            "COMMENT" => $comment,
            "CDOPERADOR" => $createdBy,
            "DOCUMENT" => $document,
        );
        return $this->requestPMRoute('/saveApproveIssueByProductCall', $this->getDataRequestRow($row));
    }

    public function giveUpImplementationIssue($issueId, $comment, $createdBy,$document = null)
    {
        $row = array(
            "ISSUE_ID" => $issueId,
            "COMMENT" => $comment,
            "CDOPERADOR" => $createdBy,
            "DOCUMENT" => $document,
        );
        return $this->requestPMRoute('/saveGiveUpImplementationIssue', $this->getDataRequestRow($row));
    }

    public function findCommentsOnIssueForProductCall($issueId)
    {
        return $this->commentsRepository->getCommentsOfTaskExceptCommentedBy($issueId);
    }

    /**
     * @param $externalCode
     * @return null|\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource
     */
    public function findResourceByExternalCode($externalCode) {
        return $this->resourceRepository->findOneBy(array('externalCode'=>$externalCode));
    }

    public function findResourceById($resourceId) {
        return $this->resourceRepository->findOneBy(array('id' => $resourceId));
    }

    public function uploadIssueAttachments($issueId, $documents, $cdOperador, $commentId = null) {
        $row = array(
            "ISSUE_ID" => $issueId,
            "DOCUMENTS" => $documents,
            "CDOPERADOR" => $cdOperador,
        );

        if($commentId){
            $row['COMMENT_ID'] = $commentId;
        }

        $dataRow = $this->getDataRequestRow($row);
        $result = $this->requestPMRoute('/uploadIssueAttachments', $dataRow);
        return $result['attachmentsIds'];
    }

    public function getIssueAttachmentsDataSet($issueId, $commentId = null) {
        $commentIdQuery = "";
        if($commentId) {
            $commentIdQuery = "AND C.ID = " . $commentId;
        }
        return $this->attachmentRepository->getTaskAttachmentsExceptUploadedBy($issueId,$this->fileServerFilesUrl, $commentIdQuery);
    }

    public function getAttachmentsDataSet($attachmentId) {
        $filter = array(
            array(
                "name" => "ID",
                "value" => $attachmentId,
                "operator" => "="
            )
        );
        $dataFilter = $this->getDataRequestFilter($filter);
        $result = $this->requestPMRoute('/getAttachmentsDataSet', $dataFilter);
        return $result['attachmentsIds'];
    }

    public function deleteAttachment($attachmentId) {
        $dataSet = array(
            array(
                'ID' => $attachmentId
            )
        );

        $dataDataSet = $this->getDataRequestDataSet($dataSet);
        $result = $this->requestPMRoute('/deleteAttachment', $dataDataSet);
        return $result;

    }

    public function setSessionPM() {
        $dataSet = array(
            array(
                'USER_ID' => 1
            )
        );

        $dataDataSet = $this->getDataRequestDataSet($dataSet);
        $result = $this->requestPMRoute('/setUserCP', $dataDataSet);
        return $result;

    }

    public function lastEvolutionDates(array $issuesIds){
        $issuesIdsQuery = implode(',', $issuesIds);
        $dateComments = $this->taskRepository->findLastCommentDateRelatedToIssuesExceptCommentedBy($issuesIdsQuery);
        $dateAppropriations = $this->taskRepository->findLastAppropriationDateRelatedToIssuesAndItsTasks($issuesIdsQuery);

        $dateComments = DevOpsUtilities::getArrayFromArrayAssociative($dateComments);
        $dateAppropriations = DevOpsUtilities::getArrayFromArrayAssociative($dateAppropriations);

        $lastDates = array();
        foreach($issuesIds as $issuesId) {
            $lastDateComment = empty($dateComments[$issuesId]['LASTDATE']) ? null : $dateComments[$issuesId]['LASTDATE'];
            $lastDateAppropriation = empty($dateAppropriations[$issuesId]['LASTDATE']) ? null : $dateAppropriations[$issuesId]['LASTDATE'];

            $lastDate = array(
                'ID' => $issuesId,
                'LASTDATE' => null
            );
            if($lastDateComment == null) {
                $lastDate['LASTDATE'] = $lastDateAppropriation;
            } else if ($lastDateAppropriation == null) {
                $lastDate['LASTDATE'] = $lastDateComment;
            } else {
                $comp = \Util\Utilities::compareDates($lastDateComment, $lastDateAppropriation);
                if($comp == 0 || $comp > 0) {
                    $lastDate['LASTDATE'] = $lastDateComment;
                } else {
                    $lastDate['LASTDATE'] =  $lastDateAppropriation;
                }
            }

            $lastDates[] = $lastDate;
        }

        return $lastDates;
    }
}
